==> hermes-5.2.0.6.tar.gz: hermes-5.2.0.6/app/models/hermes/required_document.rb <==
# frozen_string_literal: true

module Hermes
  # Il représente une pièce à fournir et contient
  # un statut (pièce manquante, contrôle en cours, pièce rejetée, pièce valide),
  # une liste des codes erreurs remontés,
  # et le lien vers le ou les fichiers.
  #
  # il est lié à un ou plusieurs applicant, ainsi,
  # une pièce peut être retrouvée et resservir pour plusieurs housingRequestFolder
  class RequiredDocument
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::Status
    
    field :nature, type: String # nature de piece à fournir
    field :type, type: String # type de piece à fournir
    field :label, type: String # 
    field :category, type: String
    field :sne_code, type: String
    field :sne_category, type: String
    field :sne_piece_label, type: String
    field :sne_piece_id, type: String
    field :folder_where_is_required_for_nud, type: Array
    field :folder_where_is_required_for_consultation, type: Array
    field :folder_where_is_required_for_application, type: Array
    field :last_upload_to_sne_date, type: Date
    field :type_ihm, type: String
    field :ihm_order, type: String
    field :or_required_document_types, type: Array, default: []
    field :declared, type: Boolean, default: nil
    
    
    belongs_to :applicant, class_name: 'Hermes::Applicant', index: true
    
    writer_auth_roles "collaborateur offre logement", 'superadministrateur'
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    before_create :before_create_action
    before_save :before_save_action
    after_create :after_create_actions
    after_save :after_save_actions
    after_destroy :after_destroy_actions
    
    RequiredDocument.add_status(:single)
    
    common_indexes *%i[
      nature
      type
      category
   
      
    ]
 
    def set_viwed_to_false
      folder_where_is_required_for_application.each do |folder_id|
        
        Hermes::HousingRequestFolder.find(folder_id)&.housing_request.set(viewed_by_colaborator_offre_logement: false, updated_at: DateTime.current)
      end
    end
  

    def concerned_unique_numbers_and_housing_request_id
      hrs = HousingRequest.any_of({main_applicant_id: applicant&.id}, {main_applicant_id: applicant&.id.to_s}, {applicant_ids: applicant&.id}, applicant_ids: applicant&.id.to_s)
      unique_numbers = []
      hrs.each do |concerned_hr|
        next unless concerned_hr.nud_ids || concerned_hr.nud_ids.empty?
        
        concerned_hr.nuds.each do |nud|
          unique_numbers.push(unique_number: nud.unique_number, housing_request_id: concerned_hr.id)
        end
      end
      unique_numbers.uniq
    end
    
    
  
    def after_create_actions;
    end
    
    def update_parent_folders_status
      parent_folder_ids = folder_where_is_required_for_nud + folder_where_is_required_for_consultation + folder_where_is_required_for_application
      parent_folder_ids.each do |folder_id|
        parent_folder = Hermes::HousingRequestFolder.find(folder_id)
        parent_folder.compute_statuses
        parent_folder.save
      end
    end
    
    def set_status
      if declared
        self.status = 'Pièce validée'
      else
          self.status = 'Pièce manquante'
      end
    end
    

    def missing?
      ['pièce manquante'].include?(status&.downcase)
    end


    
        
    
    def before_create_action
      self.writers.push(applicant.user_id)
    end
    
    def before_save_action
      set_status if   declared_changed?
    end
    
    def after_save_actions
     

      check_status_changed
      if status_changed?
        update_parent_folders_status
      end
      update_parent_folders_cache_key
    end
    
    def after_destroy_actions
      update_parent_folders_cache_key
    end

    def update_parent_folders_cache_key
      parent_folder_ids = folder_where_is_required_for_nud + folder_where_is_required_for_consultation + folder_where_is_required_for_application
      Hermes::HousingRequestFolder.where(:id.in => parent_folder_ids).set(updated_at: Time.now)
    end


  end
end

==> hermes-6.0.2.24.tar.gz: hermes-6.0.2.24/app/models/hermes/required_document.rb <==
# frozen_string_literal: true

module Hermes
  # Il représente une pièce à fournir et contient
  # un statut (pièce manquante, contrôle en cours, pièce rejetée, pièce valide),
  # une liste des codes erreurs remontés,
  # et le lien vers le ou les fichiers.
  #
  # il est lié à un ou plusieurs applicant, ainsi,
  # une pièce peut être retrouvée et resservir pour plusieurs housingRequestFolder
  class RequiredDocument
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::Status
    
    field :nature, type: String # nature de piece à fournir
    field :type, type: String # type de piece à fournir
    field :label, type: String # 
    field :category, type: String
    field :sne_code, type: String
    field :sne_category, type: String
    field :sne_piece_label, type: String
    field :sne_piece_id, type: String
    field :folder_where_is_required_for_nud, type: Array
    field :folder_where_is_required_for_consultation, type: Array
    field :folder_where_is_required_for_application, type: Array
    field :last_upload_to_sne_date, type: Date
    field :type_ihm, type: String
    field :ihm_order, type: String
    field :or_required_document_types, type: Array, default: []
    field :declared, type: Boolean, default: nil
    
    
    belongs_to :applicant, class_name: 'Hermes::Applicant', index: true
    
    writer_auth_roles "collaborateur offre logement", 'superadministrateur'
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    before_create :before_create_action
    before_save :before_save_action
    after_create :after_create_actions
    after_save :after_save_actions
    after_destroy :after_destroy_actions
    
    RequiredDocument.add_status(:single)
    
    common_indexes *%i[
      nature
      type
      category
   
      
    ]
 
    def set_viwed_to_false
      folder_where_is_required_for_application.each do |folder_id|
        
        Hermes::HousingRequestFolder.find(folder_id)&.housing_request.set(viewed_by_colaborator_offre_logement: false, updated_at: DateTime.current)
      end
    end

    def concerned_unique_numbers_and_housing_request_id
      hrs = HousingRequest.any_of({main_applicant_id: applicant&.id}, {main_applicant_id: applicant&.id.to_s}, {applicant_ids: applicant&.id}, applicant_ids: applicant&.id.to_s)
      unique_numbers = []
      hrs.each do |concerned_hr|
        next unless concerned_hr.nud_ids || concerned_hr.nud_ids.empty?
        
        concerned_hr.nuds.each do |nud|
          unique_numbers.push(unique_number: nud.unique_number, housing_request_id: concerned_hr.id)
        end
      end
      unique_numbers.uniq
    end
  
    def after_create_actions
    end
    
    def update_parent_folders_status
      parent_folder_ids = folder_where_is_required_for_nud + folder_where_is_required_for_consultation + folder_where_is_required_for_application
      parent_folder_ids.each do |folder_id|
        parent_folder = Hermes::HousingRequestFolder.find(folder_id)
        parent_folder.compute_statuses
        parent_folder.save
      end
    end
    
    def set_status
      self.status = declared ? 'Pièce validée' : 'Pièce manquante'
    end
    
    def missing?
      ['pièce manquante'].include?(status&.downcase)
    end

    def before_create_action
      self.writers.push(applicant.user_id)
    end
    
    def before_save_action
      set_status if declared_changed?
    end
    
    def after_save_actions
      check_status_changed
      if status_changed?
        update_parent_folders_status
      end
      update_parent_folders_cache_key
    end
    
    def after_destroy_actions
      update_parent_folders_cache_key
    end

    def update_parent_folders_cache_key
      parent_folder_ids = folder_where_is_required_for_nud + folder_where_is_required_for_consultation + folder_where_is_required_for_application
      Hermes::HousingRequestFolder.where(:id.in => parent_folder_ids).set(updated_at: Time.now)
    end


  end
end
